#ifndef _DEVICE_H_
#define _DEVICE_H_

// its not allowed to use extern together with _at_ 
// anyway i just want one file with buffer declarations
// so some preprocessor magic needs to do the job 
// ugly but working found this years ago in some Anchor Chips declarations
#ifdef GLOBAL_EXTERN      // define Global only in one file (Main)
  #define EXTERN          // remove EXTERN
  #define _AT_ _at_       // replace with _at_
#else                     // declare as extern
  #define EXTERN extern 
  #define _AT_ ;/ ## /    // replace "_AT_ 0xXXXX" with ';//' 
#endif

#define EP0_SIZE            8                  // 8 is just for testing could be also 16 / 32 / 64
#define EPX_SIZE            64                 // bulks are always 64 bytes
#define FREQ_SYS            24000000L          // operating at 24 MHz

#define USB_STATE_POWERED   0                  // maintaining the enum state
#define USB_STATE_RESET     1
#define USB_STATE_ADDRESSED 2
#define USB_STATE_CONFIG    3
#define USB_STATE_SUSPEND   4

#define LED_TIMEOUT         1                   //  5  ms
#define SENSE_TIMEOUT       70                  // 350 ms
  
#define MIDIBUF_SIZE        (2*EPX_SIZE)        // has to be 2^n here 128 bytes 
#define MIDIBUF_MASK        (MIDIBUF_SIZE-1)    // mask for overflow corrections
//#define MIDIBUF_LIMIT     (MIDIBUF_SIZE *2/3) // upper limit for taking action     

typedef struct _MESSAGE
{
   uint8_t Cnt;    //number of valid bytes
   uint8_t Buf[3]; //buffer
}MSG;
 
typedef struct _FIFOCTL
{
	 uint8_t wr;     // wr index
	 uint8_t rd;     // rd index
   uint8_t level;  // fillgrade
}FIFOCTL;

typedef struct _SETUP
{
    uint8_t  bRequestType;
    uint8_t  bRequest;
    uint16_t wValue;
    uint16_t wIndex;
    uint16_t wLength;
}SETUP;

//global vars declared here can be seen in any file which includes device.h
EXTERN uint8_t xdata EP0_Buffer   [EP0_SIZE]     _AT_ 0x0000; // control
EXTERN uint8_t xdata EP2_OutBuffer[EPX_SIZE]     _AT_ 0x0040; // bulk out
EXTERN uint8_t xdata EP2_InBuffer [EPX_SIZE]     _AT_ 0x0080; // bulk in
EXTERN uint8_t xdata MidiRxBuffer [MIDIBUF_SIZE] _AT_ 0x0100; // fifo rx
EXTERN uint8_t xdata MidiTxBuffer [MIDIBUF_SIZE] _AT_ 0x0200; // fifo tx

EXTERN SETUP   data Setup;      // holding the setup packet

EXTERN FIFOCTL data MidiRxCtl;  // fifos for rx and tx
EXTERN FIFOCTL data MidiTxCtl;

EXTERN bit     bTx0Empty;       // flag for signaling no more data in out fifo
EXTERN bit     bActiveSense;    // flag for signaling active sense timeout
EXTERN uint8_t rx0led;          // timeouts for activity leds
EXTERN uint8_t tx0led;
#endif