#include ".\..\inc\mytypes.h"
#include ".\..\inc\usb\usb11.h"
#include ".\..\inc\usb\usbmidi10.h"
#include ".\..\inc\device.h"

const uint8_t code DevDesc[18] =
{
   sizeof(DeviceDescriptor), USB_DEVICE_DESCRIPTOR,
   BCD_USB & 0xFF, BCD_USB >> 8,
   USB_CLASS_UNDEFINED,        // 
   USB_SUBCLASS_UNDEFINED,     // 
   USB_PROTOCOL_UNDEFINED,     // 
   EP0_SIZE,
   0x1A86 & 0xFF,0x1A86 >> 8,  // 0x86,0x1a,
   0x5724 & 0xFF,0x5724 >> 8,  // 0x24,0x57,
   0x0100 & 0xFF,0x0100 >> 8,  // Version 1.0
   0x01,                       // vendorstr
   0x02,                       // devicestr
   USB_STRING_UNDEFINED,
   1                           // 1 configuration
};

/*
Mididescriptors are identical to those from the USB 1.0 spec
Layout:
(EP2out)     (Id 1)          (Id 4)  
BulkOut -> Embeded In   -> External Out

(EP2In)      (Id 3)          (Id 2)  
BulIn   <- Embedded Out <- External In
*/
const uint8_t code CfgDesc[0x0065] =
{
    sizeof (ConfigurationDescriptor), USB_CONFIGURATION_DESCRIPTOR,
    sizeof (CfgDesc) & 0xFF,sizeof (CfgDesc) >> 8,  
    2,                              // 2 Interfaces
    1,                              // Configuration Value
    USB_STRING_UNDEFINED,           // Iconfig
    0xA0,                           // buspowered | remotewakeup
    100/2,                          // 100 mA

    // Interface 0 control
    sizeof(InterfaceDescriptor),USB_INTERFACE_DESCRIPTOR,
    0,                              // InterfaceNr 0
    0,                              // no alt.
    0,                              // no EP
    USB_CLASS_UAC,                  // Class Audio
    UAC_SUBCLASS_CONTROL,           // SubClass Control
    USB_PROTOCOL_UNDEFINED,         // no protokoll
    USB_STRING_UNDEFINED,           // no StringID
   
       9, UAC_CS_INTERFACE,         //
       UAC_AC_HEADER,               // Header Typ
       BCD_UAC & 0xFF,BCD_UAC >> 8,
       0x009 & 0xFF, 0x009 >> 8,    // 
       1,                           // 1 stream Interface
       1,                           // Interface 1 uses this CTL
    // interace 1 stream
    sizeof(InterfaceDescriptor), USB_INTERFACE_DESCRIPTOR,
    1,                              // InterfaceNr 1
    0,                              // alternate0
    2,                              // 2 Endpoints
    USB_CLASS_UAC,                  // Class Audio
    UAC_SUBCLASS_MIDISTREAM,        // SubClass Streaming
    USB_PROTOCOL_UNDEFINED,         // no protokoll
    USB_STRING_UNDEFINED,     

       7, MS_CS_INTERFACE,           //
       MS_MIDI_HEADER,               // Header Typ
       BCD_MIDI & 0xFF,BCD_MIDI >> 8,
       0x0041  & 0xFF,0x0041  >> 8,  // 
       
       // declare all Pins
       6, MS_CS_INTERFACE,           //  
       MS_MIDI_IN_JACK,              // MidiIn
       MS_EMBEDDED_JACK,             // usb in
       1,                            // ID
       USB_STRING_UNDEFINED,         // no String
       
       6, MS_CS_INTERFACE,           //
       MS_MIDI_IN_JACK,              // MidiIn
       MS_EXTERNAL_JACK,             // 
       2,                            // ID
       USB_STRING_UNDEFINED,         // no String
       
       9, MS_CS_INTERFACE,           //
       MS_MIDI_OUT_JACK,             // MidiOut
       MS_EMBEDDED_JACK,             // EMBEDDED
       3,                            // ID
       1,                            // 1 Pin
       2,                            // source is extInJack
       1,                            // source Pin
       USB_STRING_UNDEFINED,         // no String
              
       9, MS_CS_INTERFACE,           //
       MS_MIDI_OUT_JACK,             // MidiOut
       MS_EXTERNAL_JACK,             // External
       4,                            // ID
       1,                            // 1 Pin
       1,                            // source is embIn
       1,                            // source Pin
       USB_STRING_UNDEFINED,         // no String

    sizeof (EndpointDescriptor) +2, USB_ENDPOINT_DESCRIPTOR,
    0x02,                            // EP2 Out
    EP_BULK,                         // Bulk not shared
    0x0040 & 0xFF,0x0040  >> 8,
    0x00,                            //  
    0x00,                            // no refresh
    0x00,                            // no Sync

       0x05, MS_CS_ENDPOINT,
       MS_EP_GENERAL,                // 
       1,                            // one Jack
       1,                            // embIn 

    sizeof (EndpointDescriptor) +2, USB_ENDPOINT_DESCRIPTOR,
    _HOST | 0x2,                    // EP2 In
    EP_BULK,                        // Bulk not shared
    0x0040 & 0xFF,0x0040  >> 8,     //
    0x00,                           // 
    0x00,                           // no refresh
    0x00,                           // no Sync

       0x05, MS_CS_ENDPOINT,
       MS_EP_GENERAL,               // 
       1,                           // one Jack
       3                            // embOut
};

// strings
const uint8_t code LangDes[4]=
{
   sizeof(LangDes),USB_STRING_DESCRIPTOR,
   0x09,0x04
};
// this one is chinese 
const uint8_t code Manuf_Des[10]=
{
   sizeof(Manuf_Des),USB_STRING_DESCRIPTOR,
   0x5F,0x6c,
   0xCF,0x82,
   0x81,0x6c,
   0x52,0x60
};

const uint8_t code Prod_Des[22]=
{
   sizeof(Prod_Des),USB_STRING_DESCRIPTOR,
   'C',0x00,
   'H',0x00,
   '5',0x00,
   '5',0x00,
   '2',0x00,
   '-',0x00,
   'M',0x00,
   'i',0x00,
   'd',0x00,
   'i',0x00
};
